@@ -9,7 +9,7 @@
 """
 
 __author__ = "Ryan McGrath <ryan@venodesigns.net>"
-__version__ = "1.3.4"
+__version__ = "1.4"
 
 import urllib
 import urllib2
@@ -19,6 +19,7 @@ import httplib2
 import mimetypes
 import mimetools
 import re
+import inspect
 
 import oauth2 as oauth
 
@@ -46,7 +47,10 @@ except ImportError:
 			# Seriously wtf is wrong with you if you get this Exception.
 			raise Exception("Twython requires the simplejson library (or Python 2.6) to work. http://www.undefined.org/python/")
 
-class TwythonError(Exception):
+# Detect if oauth2 supports the callback_url argument to request
+OAUTH_LIB_SUPPORTS_CALLBACK = 'callback_url' in inspect.getargspec(oauth.Client.request).args
+
+class TwythonError(AttributeError):
 	"""
 		Generic error class, catch-all for most Twython issues.
 		Special cases are handled by APILimit and AuthError.
@@ -60,7 +64,7 @@ class TwythonError(Exception):
 		self.msg = msg
 		if error_code == 400:
 			raise APILimit(msg)
-	
+
 	def __str__(self):
 		return repr(self.msg)
 
@@ -73,7 +77,7 @@ class APILimit(TwythonError):
 	"""
 	def __init__(self, msg):
 		self.msg = msg
-	
+
 	def __str__(self):
 		return repr(self.msg)
 
@@ -85,13 +89,13 @@ class AuthError(TwythonError):
 	"""
 	def __init__(self, msg):
 		self.msg = msg
-	
+
 	def __str__(self):
 		return repr(self.msg)
 
 
 class Twython(object):
-	def __init__(self, twitter_token = None, twitter_secret = None, oauth_token = None, oauth_token_secret = None, headers=None):
+	def __init__(self, twitter_token = None, twitter_secret = None, oauth_token = None, oauth_token_secret = None, headers=None, callback_url=None):
 		"""setup(self, oauth_token = None, headers = None)
 
 			Instantiates an instance of Twython. Takes optional parameters for authentication and such (see below).
@@ -115,21 +119,22 @@ class Twython(object):
 		self.twitter_secret = twitter_secret
 		self.oauth_token = oauth_token
 		self.oauth_secret = oauth_token_secret
-		
+		self.callback_url = callback_url
+
 		# If there's headers, set them, otherwise be an embarassing parent for their own good.
 		self.headers = headers
 		if self.headers is None:
-			headers = {'User-agent': 'Twython Python Twitter Library v1.3'}
-		
+			self.headers = {'User-agent': 'Twython Python Twitter Library v1.3'}
+
 		consumer = None
 		token = None
-		
+
 		if self.twitter_token is not None and self.twitter_secret is not None:
 			consumer = oauth.Consumer(self.twitter_token, self.twitter_secret)
-		
+
 		if self.oauth_token is not None and self.oauth_secret is not None:
 			token = oauth.Token(oauth_token, oauth_token_secret)
-		
+
 		# Filter down through the possibilities here - if they have a token, if they're first stage, etc.
 		if consumer is not None and token is not None:
 			self.client = oauth.Client(consumer, token)
@@ -138,7 +143,7 @@ class Twython(object):
 		else:
 			# If they don't do authentication, but still want to request unprotected resources, we need an opener.
 			self.client = httplib2.Http()
-	
+
 	def __getattr__(self, api_call):
 		"""
 			The most magically awesome block of code you'll see in 2010.
@@ -152,47 +157,70 @@ class Twython(object):
 			It's called when an attribute that was called on an object doesn't seem to exist - since it doesn't exist,
 			we can take over and find the API method in our table. We then return a function that downloads and parses
 			what we're looking for, based on the keywords passed in.
-		
+
 			I'll hate myself for saying this, but this is heavily inspired by Ruby's "method_missing".
 		"""
 		def get(self, **kwargs):
 			# Go through and replace any mustaches that are in our API url.
 			fn = api_table[api_call]
 			base = re.sub(
-				'\{\{(?P<m>[a-zA-Z]+)\}\}', 
+				'\{\{(?P<m>[a-zA-Z_]+)\}\}',
 				lambda m: "%s" % kwargs.get(m.group(1), '1'), # The '1' here catches the API version. Slightly hilarious.
 				base_url + fn['url']
 			)
-			
+
 			# Then open and load that shiiit, yo. TODO: check HTTP method and junk, handle errors/authentication
 			if fn['method'] == 'POST':
-				resp, content = self.client.request(base, fn['method'], urllib.urlencode(kwargs))
+				resp, content = self.client.request(base, fn['method'], urllib.urlencode(dict([k, v.encode('utf-8')] for k, v in kwargs.items())), headers = self.headers)
 			else:
 				url = base + "?" + "&".join(["%s=%s" %(key, value) for (key, value) in kwargs.iteritems()])
-				resp, content = self.client.request(url, fn['method'])
-			
+				resp, content = self.client.request(url, fn['method'], headers = self.headers)
+
 			return simplejson.loads(content)
-		
+
 		if api_call in api_table:
 			return get.__get__(self)
 		else:
-			raise AttributeError, api_call
-	
+			raise TwythonError, api_call
+
 	def get_authentication_tokens(self):
 		"""
 			get_auth_url(self)
 
 			Returns an authorization URL for a user to hit.
 		"""
-		resp, content = self.client.request(self.request_token_url, "GET")
+		callback_url = self.callback_url or 'oob'
 		
+		request_args = {}
+		if OAUTH_LIB_SUPPORTS_CALLBACK:
+			request_args['callback_url'] = callback_url
+		
+		resp, content = self.client.request(self.request_token_url, "GET", **request_args)
+
 		if resp['status'] != '200':
 			raise AuthError("Seems something couldn't be verified with your OAuth junk. Error: %s, Message: %s" % (resp['status'], content))
 		
 		request_tokens = dict(urlparse.parse_qsl(content))
-		request_tokens['auth_url'] = "%s?oauth_token=%s" % (self.authenticate_url, request_tokens['oauth_token'])
+		
+		oauth_callback_confirmed = request_tokens.get('oauth_callback_confirmed')=='true'
+		
+		if not OAUTH_LIB_SUPPORTS_CALLBACK and callback_url!='oob' and oauth_callback_confirmed:
+			import warnings
+			warnings.warn("oauth2 library doesn't support OAuth 1.0a type callback, but remote requires it") 
+			oauth_callback_confirmed = False
+
+		auth_url_params = {
+			'oauth_token' : request_tokens['oauth_token'],
+		}
+		
+		# Use old-style callback argument
+		if callback_url!='oob' and not oauth_callback_confirmed:
+			auth_url_params['oauth_callback'] = callback_url
+		
+		request_tokens['auth_url'] = self.authenticate_url + '?' + urllib.urlencode(auth_url_params)
+		
 		return request_tokens
-	
+
 	def get_authorized_tokens(self):
 		"""
 			get_authorized_tokens
@@ -201,17 +229,17 @@ class Twython(object):
 		"""
 		resp, content = self.client.request(self.access_token_url, "GET")
 		return dict(urlparse.parse_qsl(content))
-	
+
 	# ------------------------------------------------------------------------------------------------------------------------
 	# The following methods are all different in some manner or require special attention with regards to the Twitter API.
 	# Because of this, we keep them separate from all the other endpoint definitions - ideally this should be change-able,
 	# but it's not high on the priority list at the moment.
 	# ------------------------------------------------------------------------------------------------------------------------
-	
+
 	@staticmethod
 	def constructApiURL(base_url, params):
-		return base_url + "?" + "&".join(["%s=%s" %(Twython.unicode2utf8(key), Twython.unicode2utf8(value)) for (key, value) in params.iteritems()])
-	
+		return base_url + "?" + "&".join(["%s=%s" %(Twython.unicode2utf8(key), urllib.quote_plus(Twython.unicode2utf8(value))) for (key, value) in params.iteritems()])
+
 	@staticmethod
 	def shortenURL(url_to_shorten, shortener = "http://is.gd/api.php", query = "longurl"):
 		"""shortenURL(url_to_shorten, shortener = "http://is.gd/api.php", query = "longurl")
@@ -223,33 +251,27 @@ class Twython(object):
 				shortener - In case you want to use a url shortening service other than is.gd.
 		"""
 		try:
-			resp, content = self.client.request(
-				shortener + "?" + urllib.urlencode({query: Twython.unicode2utf8(url_to_shorten)}), 
-				"GET"
-			)
+			content = urllib2.urlopen(shortener + "?" + urllib.urlencode({query: Twython.unicode2utf8(url_to_shorten)})).read()
 			return content
 		except HTTPError, e:
 			raise TwythonError("shortenURL() failed with a %s error code." % `e.code`)
-	
-	def bulkUserLookup(self, ids = None, screen_names = None, version = None):
-		""" bulkUserLookup(self, ids = None, screen_names = None, version = None)
-			
+
+	def bulkUserLookup(self, ids = None, screen_names = None, version = 1, **kwargs):
+		""" bulkUserLookup(self, ids = None, screen_names = None, version = 1, **kwargs)
+
 			A method to do bulk user lookups against the Twitter API. Arguments (ids (numbers) / screen_names (strings)) should be flat Arrays that
 			contain their respective data sets.
 
 			Statuses for the users in question will be returned inline if they exist. Requires authentication!
 		"""
-		apiURL = "http://api.twitter.com/1/users/lookup.json?lol=1"
-		if ids is not None:
-			apiURL += "&user_id="
-			for id in ids:
-				apiURL += `id` + ","
-		if screen_names is not None:
-			apiURL += "&screen_name="
-			for name in screen_names:
-				apiURL += name + ","
+		if ids:
+			kwargs['user_id'] = ','.join(map(str, ids))
+		if screen_names:
+			kwargs['screen_names'] = ','.join(screen_names)
+			
+		lookupURL = Twython.constructApiURL("http://api.twitter.com/%d/users/lookup.json" % version, kwargs)
 		try:
-			resp, content = self.client.request(apiURL, "GET")
+			resp, content = self.client.request(lookupURL, "GET", headers = self.headers)
 			return simplejson.loads(content)
 		except HTTPError, e:
 			raise TwythonError("bulkUserLookup() failed with a %s error code." % `e.code`, e.code)
@@ -266,42 +288,42 @@ class Twython(object):
 		"""
 		searchURL = Twython.constructApiURL("http://search.twitter.com/search.json", kwargs)
 		try:
-			resp, content = self.client.request(searchURL, "GET")
+			resp, content = self.client.request(searchURL, "GET", headers = self.headers)
 			return simplejson.loads(content)
 		except HTTPError, e:
 			raise TwythonError("getSearchTimeline() failed with a %s error code." % `e.code`, e.code)
-	
-	def searchTwitterGen(self, **kwargs):
+
+	def searchTwitterGen(self, search_query, **kwargs):
 		"""searchTwitterGen(search_query, **kwargs)
 
 			Returns a generator of tweets that match a specified query.
 
-			Parameters:	
+			Parameters:
 				See the documentation at http://dev.twitter.com/doc/get/search. Pass in the API supported arguments as named parameters.
 
 				e.g x.searchTwitter(q="jjndf", page="2")
 		"""
-		searchURL = Twython.constructApiURL("http://search.twitter.com/search.json", kwargs)
+		searchURL = Twython.constructApiURL("http://search.twitter.com/search.json?q=%s" % Twython.unicode2utf8(search_query), kwargs)
 		try:
-			resp, content = self.client.request(searchURL, "GET")
+			resp, content = self.client.request(searchURL, "GET", headers = self.headers)
 			data = simplejson.loads(content)
 		except HTTPError, e:
 			raise TwythonError("searchTwitterGen() failed with a %s error code." % `e.code`, e.code)
-		
+
 		if not data['results']:
 			raise StopIteration
 
 		for tweet in data['results']:
 			yield tweet
-		
+
 		if 'page' not in kwargs:
 			kwargs['page'] = 2
 		else:
 			kwargs['page'] += 1
-		
+
 		for tweet in self.searchTwitterGen(search_query, **kwargs):
 			yield tweet
-	
+
 	def isListMember(self, list_id, id, username, version = 1):
 		""" isListMember(self, list_id, id, version)
 
@@ -316,12 +338,12 @@ class Twython(object):
 				version (number) - Optional. API version to request. Entire Twython class defaults to 1, but you can override on a function-by-function or class basis - (version=2), etc.
 		"""
 		try:
-			resp, content = self.client.request("http://api.twitter.com/%d/%s/%s/members/%s.json" % (version, username, list_id, `id`))
+			resp, content = self.client.request("http://api.twitter.com/%d/%s/%s/members/%s.json" % (version, username, list_id, `id`), headers = self.headers)
 			return simplejson.loads(content)
 		except HTTPError, e:
 			raise TwythonError("isListMember() failed with a %d error code." % e.code, e.code)
 
-	def isListSubscriber(self, list_id, id, version = 1):
+	def isListSubscriber(self, username, list_id, id, version = 1):
 		""" isListSubscriber(self, list_id, id, version)
 
 			Check if a specified user (id) is a subscriber of the list in question (list_id).
@@ -335,7 +357,7 @@ class Twython(object):
 				version (number) - Optional. API version to request. Entire Twython class defaults to 1, but you can override on a function-by-function or class basis - (version=2), etc.
 		"""
 		try:
-			resp, content = "http://api.twitter.com/%d/%s/%s/following/%s.json" % (version, username, list_id, `id`)
+			resp, content = self.client.request("http://api.twitter.com/%d/%s/%s/following/%s.json" % (version, username, list_id, `id`), headers = self.headers)
 			return simplejson.loads(content)
 		except HTTPError, e:
 			raise TwythonError("isListMember() failed with a %d error code." % e.code, e.code)
@@ -348,7 +370,7 @@ class Twython(object):
 
 			Parameters:
 				image - Required. Must be a valid GIF, JPG, or PNG image of less than 800 kilobytes in size. Images with width larger than 2048 pixels will be forceably scaled down.
-				tile - Optional (defaults to true). If set to true the background image will be displayed tiled. The image will not be tiled otherwise. 
+				tile - Optional (defaults to true). If set to true the background image will be displayed tiled. The image will not be tiled otherwise.
 				** Note: It's sad, but when using this method, pass the tile value as a string, e.g tile="false"
 				version (number) - Optional. API version to request. Entire Twython class defaults to 1, but you can override on a function-by-function or class basis - (version=2), etc.
 		"""
@@ -380,7 +402,32 @@ class Twython(object):
 			return urllib2.urlopen(r).read()
 		except HTTPError, e:
 			raise TwythonError("updateProfileImage() failed with a %d error code." % e.code, e.code)
-	
+		
+	def getProfileImageUrl(self, username, size=None, version=1):
+		""" getProfileImageUrl(username)
+		
+		    Gets the URL for the user's profile image.
+		    
+		    Parameters:
+		        username - Required. User name of the user you want the image url of.
+		        size - Optional. Image size. Valid options include 'normal', 'mini' and 'bigger'. Defaults to 'normal' if not given.
+				version (number) - Optional. API version to request. Entire Twython class defaults to 1, but you can override on a function-by-function or class basis - (version=2), etc.
+		"""
+		url = "http://api.twitter.com/%s/users/profile_image/%s.json" % (version, username)
+		if size:
+			url = self.constructApiURL(url, {'size':size})
+		
+		client = httplib2.Http()
+		client.follow_redirects = False
+		resp, content = client.request(url, 'GET')
+		
+		if resp.status in (301,302,303,307):
+			return resp['location']
+		elif resp.status == 200:
+			return simplejson.loads(content)
+		
+		raise TwythonError("getProfileImageUrl() failed with a %d error code." % resp.status, resp.status)
+
 	@staticmethod
 	def encode_multipart_formdata(fields, files):
 		BOUNDARY = mimetools.choose_boundary()
@@ -402,7 +449,7 @@ class Twython(object):
 		body = CRLF.join(L)
 		content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
 		return content_type, body
-	
+
 	@staticmethod
 	def unicode2utf8(text):
 		try:
